/*
 * ui.c: 
 *
 * Tries to load the wanted UI and start it. When initializing UIs first
 * calls get_ui_func() to get the function pointers from the plugin.
 * After this is done, calls the resolved ui_main(). Now the UI takes over.
 *
 * Copyright (C) 2002 Sakari Lehtonen <sakari@ionstream.fi>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include "umix_common.h"
#include "plugin.h"
#include "ui.h"

int ui_init(const char *uiname)
{
	int retval;
	void *handle;
	ui_func *(*get_ui_func)(void);
	ui_func *ui_func_ptr;

	/* load the plugin */
	handle = load_plugin(uiname, UMIX_PLUGIN_UI);
	if( handle == NULL )
	{
		errorstr(_("Could not load %s UI plugin"), uiname); 
		return -1;
	}
	/* get the pointer to the function that returns all 
	 * functions in the ui plugin */
	get_ui_func = get_dyn_module_sym(handle, "get_ui_func");
	if( get_ui_func == NULL )
	{
		errormsg(_("get_ui_func() not found in %s UI plugin"), uiname);
		return -1;
	}	
	/* get all the ui function pointers */
	ui_func_ptr = get_ui_func();

	/* ui_deinit must be registered before starting the ui */
	atexit(ui_func_ptr->ui_deinit);
	/* the user interface takes control from here */
	retval = ui_func_ptr->ui_main();
	
	return retval;
}
