/*
 * prefs.c: 
 * GTK+ UI preferences for Umix.
 *
 * Copyright (C) 2002 Sakari Lehtonen <sakari@ionstream.fi>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <gtk/gtk.h>
#include "umix_common.h"
#include "ui.h"
#include "ui_gtk_common.h"
#include "ui_gtk_prefs.h"

enum
{
	COLUMN_VIEW = 0,
	COLUMN_NAME
};

enum
{
	FRAME_BORDER = 5
};

static GtkWidget *prefs_window = NULL;
/* check buttons */
/*
static GtkWidget *show_rec_checkbutton;
static GtkWidget *show_bal_checkbutton;
*/

static GtkWidget *dev_entry;
static GtkWidget *cfg_entry;
static GtkWidget *ui_entry;
static GtkWidget *driver_entry;
static GtkWidget *plugindir_entry;

static GtkWidget *create_global_frame(void);
static GtkWidget *create_okcancel_hbuttonbox(void);
/*
static GtkWidget *create_channels_frame(void);
*/

extern void prefs_update(void);

static void cb_prefs_ok(GtkWidget *widget, gpointer data)
{
	/* GUI tab */
	/*
	gtk_config.show_record_button = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(show_rec_checkbutton));
	gtk_config.show_balance_adj = gtk_toggle_button_get_active(
		GTK_TOGGLE_BUTTON(show_bal_checkbutton));
	*/

	/* Global tab */
	strncpy(globalopts.devpath, gtk_entry_get_text(GTK_ENTRY(dev_entry)), 
		sizeof(globalopts.devpath));
	strncpy(globalopts.cfgpath, gtk_entry_get_text(GTK_ENTRY(cfg_entry)), 
		sizeof(globalopts.cfgpath));
	strncpy(globalopts.ui, gtk_entry_get_text(GTK_ENTRY(ui_entry)), 
		sizeof(globalopts.ui));
	strncpy(globalopts.driver, gtk_entry_get_text(GTK_ENTRY(driver_entry)), 
		sizeof(globalopts.driver));
	strncpy(globalopts.plugindir, 
		gtk_entry_get_text(GTK_ENTRY(plugindir_entry)), 
		sizeof(globalopts.plugindir));

	/*
	prefs_update();
*/
	cb_prefs_delete(NULL, NULL);
	return;
}

void cb_prefs_delete(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(prefs_window);
	prefs_window = NULL;
}

void cb_show_prefs(GtkWidget *widget, gpointer data)
{
	GtkWidget *top_vbox;
	GtkWidget *notebook;
	GtkWidget *global_frame;
	GtkWidget *global_tab_label;
	/*
	GtkWidget *channels_frame;
	GtkWidget *chan_tab_label;
	*/
	GtkWidget *okcancel_hbuttonbox;
	int page = 0;

	if( prefs_window != NULL )
	{
		gdk_window_raise(prefs_window->window);
		return;
	}

	/* create the prefs window */
	prefs_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_wmclass(GTK_WINDOW(prefs_window), "prefs", "Umix");
	g_signal_connect(G_OBJECT(prefs_window), "delete_event", 
		G_CALLBACK(cb_prefs_delete), NULL);

	/* set window properties */
	gtk_window_set_title(GTK_WINDOW(prefs_window), _("Preferences - Umix"));
	gtk_window_set_resizable(GTK_WINDOW(prefs_window), FALSE);
	gtk_window_set_position(GTK_WINDOW(prefs_window), GTK_WIN_POS_MOUSE);
	gtk_container_set_border_width(GTK_CONTAINER(prefs_window), 5);
	/*gtk_window_set_default_size(GTK_WINDOW(prefs_window), WIN_W, WIN_H);*/

	/* the top_vbox contains all other widgets */
	top_vbox = gtk_vbox_new(NONHOMOGENEOUS, 10);
	gtk_container_add(GTK_CONTAINER(prefs_window), top_vbox);
	gtk_widget_show(top_vbox);

	/* create notebook */
	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(top_vbox), notebook, FILL, EXPAND, 0);
	gtk_widget_show(notebook);

	/* Global tab */
	global_frame = create_global_frame();
	gtk_container_add(GTK_CONTAINER(notebook), global_frame);
	gtk_widget_show(global_frame);

	global_tab_label = gtk_label_new ("Global");
	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook), 
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page++), 
		global_tab_label);

	/* Channels tab */
	/*
	channels_frame = create_channels_frame();
	gtk_container_add(GTK_CONTAINER(notebook), channels_frame);
	gtk_widget_show(channels_frame);

	chan_tab_label = gtk_label_new(_("Channels"));
	gtk_notebook_set_tab_label(GTK_NOTEBOOK (notebook), 
		gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page++), 
		chan_tab_label);
	gtk_widget_show(chan_tab_label);
	*/

	/* Ok and Cancel buttons */
	okcancel_hbuttonbox = create_okcancel_hbuttonbox();
	gtk_box_pack_start(GTK_BOX(top_vbox), okcancel_hbuttonbox, 
		NOEXPAND, NOFILL, 0);
	gtk_widget_show(okcancel_hbuttonbox);

	gtk_widget_show(prefs_window);
}
	
/* creates the view options frame in the GUI tab */
/*
static GtkWidget *create_view_frame(void)
{
	GtkWidget *top_frame;
	GtkWidget *checkbutton_vbox;

	top_frame = gtk_frame_new(_("View options"));
	gtk_container_set_border_width(GTK_CONTAINER(top_frame), FRAME_BORDER);

	checkbutton_vbox = gtk_vbox_new(NONHOMOGENEOUS, 0);
	gtk_container_add(GTK_CONTAINER(top_frame), checkbutton_vbox);

	show_rec_checkbutton = gtk_check_button_new_with_label(_(
			"Show record buttons"));
	gtk_box_pack_start(GTK_BOX(checkbutton_vbox), show_rec_checkbutton, 
		NOEXPAND, NOFILL, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_rec_checkbutton),
		gtk_config.show_record_button);

	show_bal_checkbutton = gtk_check_button_new_with_label(_(
			"Show balance adjustment"));
	gtk_box_pack_start(GTK_BOX(checkbutton_vbox), show_bal_checkbutton, 
		NOEXPAND, NOFILL, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_bal_checkbutton),
		gtk_config.show_balance_adj);
	
	return top_frame;
}
*/

static GtkWidget *create_entry(char *text, int limit)
{
	GtkWidget *entry;

	entry = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry), limit);
	gtk_entry_set_text(GTK_ENTRY(entry), text);
	gtk_widget_set_size_request(entry, 120, 20);
	gtk_widget_show(entry);

	return entry;
}

/* create and return the global options frame */
static GtkWidget *create_global_frame(void)
{
	GtkWidget *top_frame;
	GtkWidget *top_vbox;
	GtkWidget *table;
	GtkWidget *label;

	top_frame = gtk_frame_new(NULL);
	gtk_container_set_border_width(GTK_CONTAINER(top_frame), FRAME_BORDER);

	top_vbox = gtk_vbox_new(NONHOMOGENEOUS, 0);
	gtk_container_add(GTK_CONTAINER(top_frame), top_vbox);

	table = gtk_table_new(2, 5, NONHOMOGENEOUS);
	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
	gtk_table_set_col_spacings(GTK_TABLE(table), 2);
	gtk_box_pack_start(GTK_BOX(top_vbox), table, NOEXPAND, NOFILL, 0);
	gtk_widget_show(table);

	/* device path */
	label = gtk_label_new(_("Device path:"));
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
	gtk_widget_show(label);

	dev_entry = create_entry(globalopts.devpath, 
		sizeof(globalopts.devpath));
	gtk_table_attach_defaults(GTK_TABLE(table), dev_entry, 1, 2, 0, 1);
	gtk_widget_show(dev_entry);

	/* config path */
	label = gtk_label_new(_("Config path:"));
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
	gtk_widget_show(label);

	cfg_entry = create_entry(globalopts.cfgpath, 
		sizeof(globalopts.cfgpath));
	gtk_table_attach_defaults(GTK_TABLE(table), cfg_entry, 1, 2, 1, 2);
	gtk_widget_show(cfg_entry);
	
	/* plugin directory */
	label = gtk_label_new(_("Plugin directory:"));
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
	gtk_widget_show(label);

	plugindir_entry = create_entry(globalopts.plugindir, 
		sizeof(globalopts.plugindir));
	gtk_table_attach_defaults(GTK_TABLE(table), plugindir_entry, 
		1, 2, 2, 3);
	gtk_widget_show(plugindir_entry);
	
	/* user interface */
	label = gtk_label_new(_("User interface:"));
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
	gtk_widget_show(label);

	ui_entry = create_entry(globalopts.ui, 
		sizeof(globalopts.ui));
	gtk_table_attach_defaults(GTK_TABLE(table), ui_entry, 1, 2, 3, 4);
	gtk_widget_show(ui_entry);
	
	/* mixer driver */
	label = gtk_label_new(_("Mixer driver:"));
	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 4, 5);
	gtk_widget_show(label);

	driver_entry = create_entry(globalopts.driver, 
		sizeof(globalopts.driver));
	gtk_table_attach_defaults(GTK_TABLE(table), driver_entry, 1, 2, 4, 5);
	gtk_widget_show(driver_entry);

	gtk_widget_show(top_vbox);

	return top_frame;
}

#if 0
/* create and return the channel treemodel */
static GtkTreeModel *create_chan_model(void)
{
	GtkListStore *store;
	GtkTreeIter iter;
	int i;
	int origchan;

	store = gtk_list_store_new(2, G_TYPE_BOOLEAN, G_TYPE_STRING);
	origchan = chan_get_curr();
	for(i=0; i<mixer_get_num_chan(); i++)
	{
		chan_set_curr(i);
		gtk_list_store_append(store, &iter);
		gtk_list_store_set(store, &iter,
			0, visible_channels[0][i],
			1, chan_get_label(),
			-1);
	}
	chan_set_curr(origchan);

	return GTK_TREE_MODEL(store);
}

/* view channel togglebutton callback */
static void view_toggled_cb(GtkCellRendererToggle *cell, gchar *path_str,
	gpointer data)
{
	GtkTreeModel *model = (GtkTreeModel *)data;
	GtkTreeIter iter;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
	gboolean fixed;

	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(model, &iter, COLUMN_VIEW, &fixed, -1);

	fixed ^= 1;

	gtk_list_store_set(GTK_LIST_STORE(model), &iter, COLUMN_VIEW, 
		fixed, -1);
	gtk_tree_path_free(path);
}

/* create the frame containing channel specific options */
static GtkWidget *create_channels_frame(void)
{
	GtkWidget *top_frame;
	GtkWidget *top_vbox;
	GtkWidget *chan_sw;
	GtkWidget *treeview;
	GtkTreeModel *model;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;

	top_frame = gtk_frame_new(NULL);
	gtk_container_set_border_width(GTK_CONTAINER(top_frame), FRAME_BORDER);
	
	top_vbox = gtk_vbox_new(NONHOMOGENEOUS, 0);
	gtk_container_add(GTK_CONTAINER(top_frame), top_vbox);
	gtk_container_set_border_width(GTK_CONTAINER(top_vbox), FRAME_BORDER);
	gtk_widget_show(top_vbox);

	/* the scrolled window */
	chan_sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(chan_sw),
		GTK_SHADOW_ETCHED_IN);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(chan_sw),
		GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(top_vbox), chan_sw, EXPAND, FILL, 0);
	gtk_widget_show(chan_sw);

	model = create_chan_model();
	treeview = gtk_tree_view_new_with_model(model);
	gtk_container_add(GTK_CONTAINER(chan_sw), treeview);

	/* view column */
	renderer = gtk_cell_renderer_toggle_new();
	g_signal_connect(G_OBJECT(renderer), "toggled",
		G_CALLBACK(view_toggled_cb), model);
	column = gtk_tree_view_column_new_with_attributes(
		"View?", 
		renderer, 
		"active", 
		COLUMN_VIEW, 
		NULL);
	gtk_tree_view_column_set_sizing(GTK_TREE_VIEW_COLUMN(column),
		GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width(GTK_TREE_VIEW_COLUMN(column), 40);
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
	
	/* channel name column */
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes(
		"Channel name", 
		renderer, 
		"text", 
		COLUMN_NAME, 
		NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);

	gtk_widget_show(treeview);

	return top_frame;
}
#endif

/* creates the ok and cancel buttons at the bottom */
static GtkWidget *create_okcancel_hbuttonbox(void)
{
	GtkWidget *ok_button;
	GtkWidget *cancel_button;
	GtkWidget *top_hbuttonbox;

	top_hbuttonbox = gtk_hbutton_box_new();
	
	ok_button = gtk_button_new_from_stock(GTK_STOCK_OK);
	g_signal_connect(G_OBJECT(ok_button), "clicked",
		G_CALLBACK(cb_prefs_ok), NULL);
	gtk_container_add(GTK_CONTAINER (top_hbuttonbox), ok_button);
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
	gtk_widget_show(ok_button);

	cancel_button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
	g_signal_connect(G_OBJECT(cancel_button), "clicked",
		G_CALLBACK(cb_prefs_delete), NULL);
	gtk_container_add(GTK_CONTAINER(top_hbuttonbox), cancel_button);
	GTK_WIDGET_SET_FLAGS (cancel_button, GTK_CAN_DEFAULT);
	gtk_widget_show(cancel_button);
	
	gtk_button_box_set_layout(GTK_BUTTON_BOX(top_hbuttonbox), 
		GTK_BUTTONBOX_END);

	return top_hbuttonbox;
}
